<?php

namespace App\Http\Controllers;

use App\Models\Blog;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class BlogController extends Controller
{
    public function index()
    {
        $blogs = Blog::all();
        return view('admin.blogs.index', compact('blogs'));
    }

    public function create()
    {
        return view('admin.blogs.create');
    }

    public function store(Request $request)
    {
        // Validate request data
        $request->validate([
            'heading' => 'required|string|max:255',
            'meta_title' => 'required|string|max:255',
            'meta_description' => 'nullable|string',
            'meta_keywords' => 'nullable|string',
            'popular' => 'required|boolean',
            'description' => 'required|string',
            'slug' => 'nullable|string|max:255|unique:blogs,slug',  // Slug validation: ensure it's unique
            'main_image' => 'nullable|image|max:2048',  // 2MB Max file size for main image
            'banner_image' => 'nullable|image|max:2048' // 2MB Max file size for banner image
        ]);

        // Log the start of the store process
        Log::info("Storing new blog");

        // Create a new blog instance
        $blog = new Blog();
        $blog->heading = $request->heading;
        $blog->meta_title = $request->meta_title;
        $blog->meta_description = $request->meta_description;
        $blog->meta_keywords = $request->meta_keywords;
        $blog->popular = $request->popular;
        $blog->description = $request->description;

        // Handling the manual slug or auto-generating it
        if ($request->has('slug') && !empty($request->slug)) {
            $blog->slug = $request->slug;  // Use the manually entered slug if provided
        } else {
            // If no manual slug is provided, auto-generate it from the heading
            $blog->slug = Str::slug($request->heading, '-');
        }

        // Log the slug that will be saved
        Log::info('Slug set to: ' . $blog->slug);

        // Handling main image upload
        if ($request->hasFile('main_image')) {
            Log::info('Main image found, processing upload');

            // Upload the new main image
            $main_image = $request->file('main_image');
            $main_filename = 'main_' . time() . '.' . $main_image->getClientOriginalExtension();
            $main_destinationPath = public_path('uploads/blogs');
            $main_image->move($main_destinationPath, $main_filename);
            $blog->main_image = 'uploads/blogs/' . $main_filename;

            Log::info('Main image uploaded and stored as: ' . $blog->main_image);
        }

        // Handling banner image upload
        if ($request->hasFile('banner_image')) {
            Log::info('Banner image found, processing upload');

            // Upload the new banner image
            $banner_image = $request->file('banner_image');
            $banner_filename = 'banner_' . time() . '.' . $banner_image->getClientOriginalExtension();
            $banner_destinationPath = public_path('uploads/blogs');
            $banner_image->move($banner_destinationPath, $banner_filename);
            $blog->banner_image = 'uploads/blogs/' . $banner_filename;

            Log::info('Banner image uploaded and stored as: ' . $blog->banner_image);
        }

        // Save the new blog
        $blog->save();

        // Log the successful creation
        Log::info('Blog created successfully with ID: ' . $blog->id);

        // Redirect with a success message
        return redirect()->route('admin.blogs.index')->with('success', 'Blog created successfully.');
    }


    public function show(Blog $blog)
    {
        return view('admin.blogs.show', compact('blog'));
    }

    public function edit(Blog $blog)
    {
        return view('admin.blogs.edit', compact('blog'));
    }

    public function update(Request $request, $id)
    {
        // Log the start of the update process with the blog ID
        Log::info("Updating blog with ID: $id");

        // Find the blog by ID
        $blog = Blog::findOrFail($id);

        // Validate request data
        $request->validate([
            'heading' => 'required|string|max:255',
            'meta_title' => 'required|string|max:255',
            'meta_description' => 'nullable|string',
            'meta_keywords' => 'nullable|string',
            'popular' => 'required|boolean',
            'description' => 'required|string',
            'slug' => 'nullable|string|max:255|unique:blogs,slug,' . $blog->id, // Slug validation, ensuring uniqueness
            'main_image' => 'nullable|image|max:2048',  // 2MB Max file size
            'banner_image' => 'nullable|image|max:2048' // 2MB Max file size
        ]);

        Log::info('Validation passed, updating blog data');

        // Update basic blog information
        $blog->heading = $request->heading;
        $blog->meta_title = $request->meta_title;
        $blog->meta_description = $request->meta_description;
        $blog->meta_keywords = $request->meta_keywords;
        $blog->popular = $request->popular;
        $blog->description = $request->description;

        // Handling the manual slug
        if ($request->has('slug') && !empty($request->slug)) {
            $blog->slug = $request->slug;  // Use the manually entered slug if provided
        } else {
            // If no manual slug is provided, auto-generate it from the heading
            $blog->slug = Str::slug($request->heading, '-');
        }

        Log::info('Slug set to: ' . $blog->slug);  // Log the slug that will be saved

        // Handling main image upload
        if ($request->hasFile('main_image')) {
            Log::info('Main image found, processing upload');

            // Delete old image if it exists
            if ($blog->main_image && file_exists(public_path($blog->main_image))) {
                Log::info('Deleting old main image: ' . $blog->main_image);
                unlink(public_path($blog->main_image));  // Delete old image
            }

            // Upload new image
            $main_image = $request->file('main_image');
            $main_filename = 'main_' . time() . '.' . $main_image->getClientOriginalExtension();
            $main_destinationPath = public_path('uploads/blogs');
            $main_image->move($main_destinationPath, $main_filename);
            $blog->main_image = 'uploads/blogs/' . $main_filename;

            Log::info('Main image uploaded and stored as: ' . $blog->main_image);
        }

        // Handling banner image upload
        if ($request->hasFile('banner_image')) {
            Log::info('Banner image found, processing upload');

            // Delete old image if it exists
            if ($blog->banner_image && file_exists(public_path($blog->banner_image))) {
                Log::info('Deleting old banner image: ' . $blog->banner_image);
                unlink(public_path($blog->banner_image));  // Delete old image
            }

            // Upload new banner image
            $banner_image = $request->file('banner_image');
            $banner_filename = 'banner_' . time() . '.' . $banner_image->getClientOriginalExtension();
            $banner_destinationPath = public_path('uploads/blogs');
            $banner_image->move($banner_destinationPath, $banner_filename);
            $blog->banner_image = 'uploads/blogs/' . $banner_filename;

            Log::info('Banner image uploaded and stored as: ' . $blog->banner_image);
        }

        // Save the updated blog data
        $blog->save();

        // Log the successful update
        Log::info('Blog updated successfully with ID: ' . $blog->id);

        // Redirect with a success message
        return redirect()->route('admin.blogs.index')->with('success', 'Blog updated successfully.');
    }



    public function destroy(Blog $blog)
    {
        if ($blog->image) {
            Storage::delete($blog->image);
        }
        $blog->delete();
        return redirect()->route('admin.blogs.index')->with('success', 'Blog deleted successfully.');
    }
}
