// ======== Utility to get device type ========
function getDeviceType() {
  const width = window.innerWidth;
  if (width <= 768) return "mobile";
  if (width <= 1024) return "tablet";
  return "pc";
}

// ======== Configuration Objects for Animations ========
const clientLogoPositions = {
  mobile: {
    start: "top 4%",
    end: "bottom -300%",
    row1: "550%",
    row2: "-550%",
  },
  tablet: {
    start: "top 6%",
    end: "bottom -350%",
    row1: "530vw",
    row2: "-530vw",
  },
  pc: {
    start: "top -1%",
    end: "bottom -205%",
    row1: "140vw",
    row2: "-140vw",
  },
};

const serviceSectionPositions = {
  mobile: "bottom -180%",
  tablet: "bottom -230%",
  pc: "bottom -280%",
};

const testimonialPositions = {
  mobile: {
    start: "top -1%",
    end: "bottom -405%",
    xMove: "-990%",
  },
  tablet: {
    start: "top -1%",
    end: "bottom -340%",
    xMove: "-360%",
  },
  pc: {
    start: "top -1%",
    end: "bottom -300%",
    xMove: "-360%",
  },
};

document.addEventListener("DOMContentLoaded", () => {
  const deviceType = getDeviceType();

  // ======== Heading Span Animation ========
  const headingSpans = document.querySelectorAll(".heading_span div span");
  if (headingSpans.length) {
    gsap.fromTo(
      headingSpans,
      { opacity: 0, y: "100px", rotate: 25, skewY: 18 },
      {
        opacity: 1,
        y: "0px",
        rotate: 0,
        skewY: 0,
        duration: 1,
        ease: "power3.out",
        stagger: 0.07,
      }
    );
  }

  // ======== Jelly Button Animation ========
  const jellyButtons = document.querySelectorAll(".jelly_btn");
  if (jellyButtons.length) {
    jellyButtons.forEach((button) => {
      button.addEventListener("mousemove", (e) => {
        const rect = button.getBoundingClientRect();
        const mouseX = e.clientX - rect.left - rect.width / 2;
        const mouseY = e.clientY - rect.top - rect.height / 2;

        gsap.to(button, {
          x: mouseX * 0.2,
          y: mouseY * 0.2,
          scale: 1.1,
          duration: 0.3,
          ease: "power3.out",
        });
      });

      button.addEventListener("mouseleave", () => {
        gsap.to(button, {
          x: 0,
          y: 0,
          scale: 1,
          duration: 0.3,
          ease: "power3.out",
        });
      });
    });
  }

  // ======== Clients Logo Animation ========
  const row1 = document.querySelector(".logo_row_1");
  const row2 = document.querySelector(".logo_row_2");
  if (row1 && row2 && clientLogoPositions[deviceType]) {
    const {
      start,
      end,
      row1: row1X,
      row2: row2X,
    } = clientLogoPositions[deviceType];
    gsap
      .timeline({
        scrollTrigger: {
          trigger: "#clientsLogo",
          start: start,
          end: end,
          scrub: 1,
          pin: true,
          pinSpacing: true,
          anticipatePin: 1,
        },
      })
      .to(row1, { x: row1X }, 0)
      .to(row2, { x: row2X }, 0);
  }

  // ======== Heartbeat Animation ========
  const heart = document.querySelector(".circle_client svg");
  if (heart) {
    gsap.to(heart, {
      scale: 1.2,
      repeat: -1,
      yoyo: true,
      duration: 0.5,
      ease: "power1.inOut",
    });
  }

  // ======== Service Section Animation ========
  const serviceSection = document.querySelector(".service_section");
  if (serviceSection && serviceSectionPositions[deviceType]) {
    gsap
      .timeline({
        scrollTrigger: {
          trigger: ".service_section",
          start: "top top",
          end: serviceSectionPositions[deviceType],
          scrub: true,
          pin: true,
          pinSpacing: true,
          anticipatePin: 1,
        },
      })
      .to(".layer_1", { scale: 10, ease: "none" }, 0)
      .to(".layer_2", { scale: 9.9, ease: "none" }, 0.004)
      .to(".layer_3", { scale: 9.8, ease: "none" }, 0.008)
      .to(".layer_4", { scale: 9.7, ease: "none" }, 0.018)
      .to("#serviceText", { scale: 1, opacity: 1, ease: "none" }, "-=0.53")
      .to("#serviceText", { opacity: 0, ease: "none" }, "+=0.1");
  }

  // ======== Testimonial Section Animation ========
  const beloved = document.querySelector(".testimonial_title_overly");
  if (beloved && testimonialPositions[deviceType]) {
    const { start, end, xMove } = testimonialPositions[deviceType];
    gsap
      .timeline({
        scrollTrigger: {
          trigger: ".testimonial_section",
          start: start,
          end: end,
          scrub: 1,
          pin: true,
          pinSpacing: true,
          anticipatePin: 1,
        },
      })
      .to(beloved, { width: "100%", ease: "none" }, 0)
      .to(".testimonial_parent", { x: xMove }, 0)
      .to(
        ".testimonial_parent .test_card:nth-child(even)",
        { marginBottom: "200px" },
        0
      )
      .to(
        ".testimonial_parent .test_card:nth-child(odd)",
        { marginTop: "200px" },
        0
      );
  }

  // ======== Purple Cover Intersection Observer Animation ========
  const purpleCovers = document.querySelectorAll(".purple_cover");
  if (purpleCovers.length) {
    const purpleObserver = new IntersectionObserver((entries) => {
      entries.forEach((entry) => {
        if (entry.isIntersecting) {
          entry.target.style.transform = "translateX(100%)";
          purpleObserver.unobserve(entry.target);
        }
      });
    });

    purpleCovers.forEach((cover) => purpleObserver.observe(cover));
  }

  // ======== Footer Links Hover Animation ========
  const footerLinks = document.querySelectorAll(".footer_links li");
  if (footerLinks.length) {
    footerLinks.forEach((li) => {
      const img = li.querySelector("img");
      if (!img) return;

      li.addEventListener("mouseenter", () => {
        gsap.to(img, {
          duration: 0.1,
          opacity: 1,
          scale: 1,
          rotate: 0,
          ease: "power2.out",
        });
      });

      li.addEventListener("mouseleave", () => {
        gsap.to(img, {
          duration: 0.1,
          opacity: 0,
          scale: 0.8,
          ease: "power2.in",
        });
      });

      li.addEventListener("mousemove", (e) => {
        const rect = li.getBoundingClientRect();
        const x = e.clientX - rect.left;
        const y = e.clientY - rect.top;
        gsap.to(img, {
          duration: 0.1,
          x: x - img.offsetWidth,
          y: y - img.offsetHeight,
          ease: "none",
        });
      });
    });
  }

  // ======== Banner Title Text Animation ========
  const bannerTitles = document.querySelectorAll(".banner_title");
  if (bannerTitles.length) {
    bannerTitles.forEach((title) => {
      const text = title.textContent;
      title.textContent = "";

      text.split("").forEach((char) => {
        const span = document.createElement("span");
        span.textContent = char === " " ? "\u00A0" : char;
        title.appendChild(span);
      });

      const spans = title.querySelectorAll("span");
      gsap.fromTo(
        spans,
        { opacity: 0, y: "100px", rotate: 25, skewY: 18 },
        {
          opacity: 1,
          y: "0px",
          rotate: 0,
          skewY: 0,
          duration: 1,
          ease: "power3.out",
          stagger: 0.07,
        }
      );
    });
  }

  // ======== SVG Intersection Observer Animation ========
  const svgs = document.querySelectorAll(".animated_svg");
  if (svgs.length) {
    const svgObserver = new IntersectionObserver(
      (entries) => {
        entries.forEach((entry) => {
          if (entry.isIntersecting) {
            const svg = entry.target;
            const paths = svg.querySelectorAll("path");
            paths.forEach((path) => {
              const length = path.getTotalLength();
              path.style.strokeDasharray = length;
              path.style.strokeDashoffset = length;
              path.classList.add("animated");
            });
            svgObserver.unobserve(svg);
          }
        });
      },
      { threshold: 0.1 }
    );

    svgs.forEach((svg) => svgObserver.observe(svg));
  }

  // ======== Smooth Counter + Intersection Observer ========
  function animateCounter(element, start, end, duration) {
    const range = end - start;
    let startTime = null;

    function step(timestamp) {
      if (!startTime) startTime = timestamp;
      const elapsed = timestamp - startTime;
      const progress = Math.min(elapsed / duration, 1);
      const currentValue = start + range * progress;

      element.textContent = Number.isInteger(end)
        ? Math.floor(currentValue)
        : currentValue.toFixed(1);

      if (progress < 1) {
        requestAnimationFrame(step);
      }
    }

    requestAnimationFrame(step);
  }

  const counters = document.querySelectorAll(".number_span");
  if (counters.length) {
    const counterObserver = new IntersectionObserver(
      (entries) => {
        entries.forEach((entry) => {
          if (entry.isIntersecting) {
            const counter = entry.target;
            const targetNumber = parseFloat(counter.textContent);
            counter.textContent = "0";
            animateCounter(counter, 0, targetNumber, 2000);
            counterObserver.unobserve(counter);
          }
        });
      },
      { threshold: 0.5 }
    );

    counters.forEach((counter) => counterObserver.observe(counter));
  }

  // ======== Sticky Sections Blur Logic ========
  const stickyParents = document.querySelectorAll(".sticky_parent");
  if (stickyParents.length) {
    window.addEventListener("scroll", () => {
      let activeIndex = -1;
      const isMobile = window.innerWidth <= 768;
      const viewportFraction = isMobile ? 1.3 : 1;

      stickyParents.forEach((section, index) => {
        const rect = section.getBoundingClientRect();
        if (
          rect.top < window.innerHeight / viewportFraction &&
          rect.bottom > window.innerHeight / viewportFraction
        ) {
          activeIndex = index;
        }
      });

      stickyParents.forEach((section, index) => {
        const card = section.querySelector(".service_card");
        if (!card) return;

        if (index < activeIndex) {
          card.classList.add("blurred");
        } else {
          card.classList.remove("blurred");
        }
      });
    });
  }

  // ======== Line Fill Animation ========
  const lineFill = document.querySelector(".line_fill");
  if (lineFill) {
    gsap.to(".line_fill", {
      scrollTrigger: {
        trigger: ".line_fill",
        start: "top 75%",
        end: "bottom 25%",
        scrub: true,
      },
      backgroundColor: "#eb3678",
    });
  }

  // ======== Pinned Process Section (MatchMedia) ========
  const pinnedProcess = document.querySelector("#pinnedProcess");
  if (pinnedProcess) {
    let mm = gsap.matchMedia();

    // ========== DESKTOP (>= 1025px) ==========
    mm.add("(min-width: 1025px)", () => {
      const tl1 = gsap.timeline({
        scrollTrigger: {
          trigger: "#pinnedProcess",
          start: "50% 50%",  // Desktop start
          end: "bottom -795%", // Desktop end
          scrub: 1,
          pin: true,
          pinSpacing: true,
          anticipatePin: 1,
        },
      });

      tl1.to(".up_layer, .down_layer", {
        height: 0,
        duration: 3,
      });

      tl1.to(".circle_points_row", {
        x: "-230%", // Desktop x value
        duration: 10,
      });

      tl1.to(
        ".circle_points_row .process_card:nth-child(odd)",
        {
          marginTop: "200px",
          duration: 3,
          stagger: 0.2,
          ease: "power1.inOut",
        },
        "<"
      );

      tl1.to(".tools_section", {
        opacity: 1,
        duration: 3,
        ease: "power1.inOut",
      });

      tl1.to(".design_process", {
        backgroundColor: "#3d0178",
        duration: 3,
        ease: "power1.inOut",
      });
    });

    // ========== TABLET (768px <= width <= 1024px) ==========
    mm.add("(min-width: 768px) and (max-width: 1024px)", () => {
      const tl1 = gsap.timeline({
        scrollTrigger: {
          trigger: "#pinnedProcess",
          start: "50% 50%",
          end: "bottom -395%",
          scrub: 1,
          pin: true,
          pinSpacing: true,
          anticipatePin: 1,
        },
      });

      tl1.to(".up_layer, .down_layer", {
        height: 0,
        duration: 3,
      });

      tl1.to(".circle_points_row", {
        x: "-290%", // Adjust for tablet
        duration: 5,
      });

      tl1.to(
        ".circle_points_row .process_card:nth-child(odd)",
        {
          marginTop: "200px",
          duration: 3,
          stagger: 0.2,
          ease: "power1.inOut",
        },
        "<"
      );

      tl1.to(".tools_section", {
        opacity: 1,
        duration: 3,
        ease: "power1.inOut",
      });

      tl1.to(".design_process", {
        backgroundColor: "#3d0178",
        duration: 3,
        ease: "power1.inOut",
      });
    });

    // ========== MOBILE (<= 767px) ==========
    mm.add("(max-width: 767px)", () => {
      const tl1 = gsap.timeline({
        scrollTrigger: {
          trigger: "#pinnedProcess",
          start: "50% 50%",
          end: "bottom -395%",
          scrub: 1,
          pin: true,
          pinSpacing: true,
          anticipatePin: 1,
        },
      });

      tl1.to(".up_layer, .down_layer", {
        height: 0,
        duration: 3,
      });

      tl1.to(".circle_points_row", {
        x: "-650%", // Adjust for mobile
        duration: 5,
      });

      tl1.to(
        ".circle_points_row .process_card:nth-child(odd)",
        {
          marginTop: "100px",
          duration: 3,
          stagger: 0.2,
          ease: "power1.inOut",
        },
        "<"
      );

      tl1.to(".tools_section", {
        opacity: 1,
        duration: 3,
        ease: "power1.inOut",
      });

      tl1.to(".design_process", {
        backgroundColor: "#3d0178",
        duration: 3,
        ease: "power1.inOut",
      });
    });
  }
});